$LOAD_PATH.push(File.dirname(__FILE__)+"/lib") 
require 'display_triangle.rb'
require 'pattern_plotter.rb'
require 'gerber_reader.rb'
require 'gcode_formatter.rb'

# --------------------------------------------------
# --- IN/OUT file

OUTPUTDIR = "output/"

data_directory = ARGV[0]+'/'
solder_pattern = data_directory+'sol.gbr'
gaikei_data = data_directory+'gaikei.gbr'
drill_data = data_directory+'ana.drl'

printf("garbar directory: %s\n", data_directory)


# --------------------------------------------------
# --- preference

ROUND_RESOLUTION = 24
PATTERN_CUTTER_SIZE = 0.24
PATTERN_FEED_SPEED = 30.0
PATTERN_CARVE_DEPTH = -0.10
CUTTING_FEED_SPEED = 80.0
CUTTING_LAYER_DEPTH = -0.4
CUTTING_THIN = -1.65

# --------------------------------------------------

graphic = MYGLWindowViewTriangle.new(800, 600, "preview")
canvas = PatternPlotter.new
canvas.set_round_resolution((ROUND_RESOLUTION/2).to_i)
canvas.marge_outline = true

gerber = GFileReader.new
gerber.load( solder_pattern )

max = gerber.get.size
p_outline = []

# --- pass 1
count = 0
gerber.get.each do |item|
  count += 1
  printf("1: %d/%d\n", count, max)
  canvas.set_mil_size(PATTERN_CUTTER_SIZE)
  canvas.set_pad_round(item['is_round'])
  canvas.set_pad_size(item['width'])
  p1 = PPoint.new(item['start_x'], item['start_y'], 0)
  p2 = PPoint.new(item['end_x'], item['end_y'], 0)
  if (p1.x == p2.x) && (p1.y == p2.y) then
    canvas.flash(p1.x, p1.y)
  else
    canvas.move(p1.x, p1.y, p2.x, p2.y)
  end
end

canvas.get_outline.each do |line|
  graphic.add_line(line)
  p_outline << line
end

# --- pass 2
count = 0
gerber.get.each do |item|
  count += 1
  printf("2: %d/%d\n", count, max)
  canvas.set_mil_size(PATTERN_CUTTER_SIZE + (PATTERN_CUTTER_SIZE*0.6))
  canvas.set_pad_round(item['is_round'])
  canvas.set_pad_size(item['width'])
  p1 = PPoint.new(item['start_x'], item['start_y'], 0)
  p2 = PPoint.new(item['end_x'], item['end_y'], 0)
  if (p1.x == p2.x) && (p1.y == p2.y) then
    canvas.flash(p1.x, p1.y)
  else
    canvas.move(p1.x, p1.y, p2.x, p2.y)
  end
end

canvas.get_object.each do |tri|
  graphic.add_object(tri)
end
canvas.get_outline.each do |line|
  graphic.add_line(line)
  p_outline << line
end


# --- gaikei
gaikei = GCodeFormatter.new
gerber = GFileReader.new
gerber.load( gaikei_data )
gerber.get.each do |item|
  gaikei.add_line(item['start_x'], item['start_y'],
                  item['end_x'], item['end_y'])
end
gaikei.mil_feed = CUTTING_FEED_SPEED
gaikei.float_height = 1.0
gaikei.mil_depth = CUTTING_THIN
gaikei.layer_height = CUTTING_LAYER_DEPTH
gaikei.calc_offset
gaikei.save(OUTPUTDIR+"t_gaikei.ncd")

# --- pattern
gcf = GCodeFormatter.new

p_outline.each do |line|
  gcf.add_line(line[0].x, line[0].y, line[1].x, line[1].y)
end
gcf.offset_x = gaikei.offset_x
gcf.offset_y = gaikei.offset_y
gcf.mil_feed = PATTERN_FEED_SPEED
gcf.float_height = 0.5
gcf.mil_depth = PATTERN_CARVE_DEPTH
gcf.save(OUTPUTDIR+"t_pattrn.ncd")

# --- drill
drl = GCodeDrill.new
drl.offset_x = gaikei.offset_x
drl.offset_y = gaikei.offset_y
drl.float_height = 0.5
drl.load( drill_data )
drl.save(OUTPUTDIR+"t_drl.ncd")

graphic.running
