require 'draw_primitive.rb'
require 'otk_gl_window.rb'

class MYGLWindowViewTriangle < MyGLWindowBase
  def initialize(width, height, title)
    super
    @triangle_object = []
    @line_object = []
    @point_object = []
    @odd = false
    @min_x = 0
    @max_x = 0
    @min_y = 0
    @max_y = 0
    @eye_home_distance = 0
    @eye_home_x = 0
    @eye_home_y = 0
  end

  def clear_object()
    @triangle_object.clear()
    @line_object.clear()
    @point_object.clear()
    @min_x = 0
    @max_x = 0
    @min_y = 0
    @max_y = 0
  end

  def running
    d1 = @max_x - @min_x
    d2 = @max_y - @min_y
    if d1 > d2 then
      @eye_distance = d1 * 1.2
    else
      @eye_distance = d2 * 1.2
    end
    @eye_target_x = (@max_x + @min_x) / 2.0
    @eye_target_y = (@max_y + @min_y) / 2.0
    @eye_home_distance = @eye_distance
    @eye_home_x = @eye_target_x
    @eye_home_y = @eye_target_y
    super
  end

  def add_object( triangle )
    @triangle_object << triangle
    check_minmax(triangle.get_point(0))
    check_minmax(triangle.get_point(1))
    check_minmax(triangle.get_point(2))
  end

  def add_line( line )
    @line_object << line
    check_minmax(line[0])
    check_minmax(line[1])
  end

  def add_point( point )
    @point_object << point
    check_minmax(point)
  end

  def draw
    GL.Disable(EnableCap.DepthTest)
    @triangle_object.each do |obj|
      draw_gl_triangle(obj) if obj.ok?
    end
    @line_object.each do |obj|
      draw_gl_line(obj) if obj.ok?
    end
    @point_object.each do |obj|
      draw_gl_point(obj)
    end
    GL.Enable(EnableCap.DepthTest)
  end

  def draw_gl_triangle(obj)
    p1 = obj.get_point(0)
    p2 = obj.get_point(1)
    p3 = obj.get_point(2)
#    GL.Color4(0.5, 1.0, 0.7, 0.5)
    GL.Color4(0.5, 1.0, 0.7, 1.0)
#    GL.Enable(EnableCap.Blend)
    GL.BlendFunc(BlendingFactorSrc.SrcAlpha, BlendingFactorDest.OneMinusSrcAlpha)
    GL.Begin(BeginMode.Triangles)
    GL.vertex3(p1.x, p1.y, p1.z)
    GL.vertex3(p2.x, p2.y, p2.z)
    GL.vertex3(p3.x, p3.y, p3.z)
    GL.end
    GL.Disable(EnableCap.Blend)
#    GL.Color3(Color.Cyan)
#    GL.Begin(BeginMode.LineLoop)
#    GL.vertex3(p1.x, p1.y, p1.z)
#    GL.vertex3(p2.x, p2.y, p2.z)
#    GL.vertex3(p3.x, p3.y, p3.z)
#    GL.end
  end

  def draw_gl_line(obj)
    p1 = obj.get_point(0)
    p2 = obj.get_point(1)
    GL.Color3(Color.Cyan)
    GL.Begin(BeginMode.Lines)
    GL.vertex3(p1.x, p1.y, p1.z)
    GL.vertex3(p2.x, p2.y, p2.z)
    GL.end
  end

  def draw_gl_point(obj)
    GL.Color3(Color.Red)
    GL.PointSize(5.0)
    GL.Begin(BeginMode.Points)
    GL.vertex3(obj.x, obj.y, obj.z)
    GL.end
  end

  # --------------------------------------
  # --- mouse operate
  # --------------------------------------

  def mouse_down(e)
    @mouse_pos_x = @window.mouse.x
    @mouse_pos_y = @window.mouse.y
    if e.Button.to_s == 'Middle' then
      @eye_target_x = @eye_home_x
      @eye_target_y = @eye_home_y
      @eye_distance = @eye_home_distance
      @refresh = true
    end
  end

  def mouse_up(e)
  end

  def mouse_move
    return if @mouse_drag == false
    mx = @window.mouse.x
    my = @window.mouse.y
    dx = (@mouse_pos_x - mx) / (512.0 / @eye_distance)
    dy = (@mouse_pos_y - my) / (512.0 / @eye_distance)
    @eye_target_x = @eye_target_x + dx
    @eye_target_y = @eye_target_y - dy
    @mouse_pos_x = mx
    @mouse_pos_y = my
    @refresh = true
  end

  def mouse_change_wheel
    move = @window.mouse.WheelPrecise
    wd = move - @mouse_wheel
    @eye_distance += (wd / 2)
    if @eye_distance < 0.5 then
      @eye_distance = 0.5
    end
    if @eye_distance > 1000.0 then
      @eye_distance = 1000.0
    end
    @mouse_wheel = move
    @refresh = true
  end

  # ---------------------------------------------
  # --- model area

  def check_minmax(point)
    if @min_x > point.x then
      @min_x = point.x
    end
    if @max_x < point.x then
      @max_x = point.x
    end
    if @min_y > point.y then
      @min_y = point.y
    end
    if @max_y < point.y then
      @max_y = point.y
    end
  end

end
