require 'draw_primitive.rb'

class GCodeFormatter

  def initialize
    @linedata = []
    @seqdata = []
    @offset_x = 0.0
    @offset_y = 0.0
    @float_height = 2.0
    @layer_height = -0.4
    @mil_depth = -0.1
    @mil_feed = 50
    @position_x = 0.0
    @position_y = 0.0
    @position_z = 0.0
    @is_build = false
    @is_home = true
  end

  attr_accessor :offset_x
  attr_accessor :offset_y
  attr_accessor :float_height
  attr_accessor :mil_depth
  attr_accessor :mil_feed
  attr_accessor :layer_height

  def clear
    @linedata.clear
    @seqdata.clear
    @offset_x = 0.0
    @offset_y = 0.0
    @float_height = 10.0
    @position_x = 0.0
    @position_y = 0.0
    @is_build = false
  end

  def add_line(x1, y1, x2, y2)
    pos = {}
    pos['x1'] = x1
    pos['y1'] = y1
    pos['x2'] = x2
    pos['y2'] = y2
    @linedata << pos
    @is_build = false
  end

  def calc_offset
    @offset_x = 10000000.0
    @offset_y = 10000000.0
    @linedata.each do |item|
      if item['x1'] < @offset_x then
        @offset_x = item['x1']
      end
      if item['x2'] < @offset_x then
        @offset_x = item['x2']
      end
      if item['y1'] < @offset_y then
        @offset_y = item['y1']
      end
      if item['y2'] < @offset_y then
        @offset_y = item['y2']
      end
    end
  end

  def match_line(lineary, x, y)
    result = -1
    for i in 0...lineary.size
      item = lineary.at(i)
      if ((item['x1'] == x) && (item['y1'] == y)) ||
         ((item['x2'] == x) && (item['y2'] == y)) then
        result = i
        break
      end
    end
    return result
  end

  def nearest_line(lineary, x, y)
    result = -1
    near = 100000000.0
    for i in 0...lineary.size
      item = lineary.at(i)
      vec1 = PPoint.new(item['x1']-x, item['y1']-y)
      if vec1.length < near then
        result = i
        near = vec1.length
      end
      vec2 = PPoint.new(item['x2']-x, item['y2']-y)
      if vec2.length < near then
        result = i
        near = vec2.length
      end
    end
    return result
  end

  def align_line(lineitem, x, y)
    match = false
    if ((lineitem['x1'] == x) && (lineitem['y1'] == y)) ||
       ((lineitem['x2'] == x) && (lineitem['y2'] == y)) then
      match = true
      if (lineitem['x2'] == x) && (lineitem['y2'] == y) then
        tx = lineitem['x1']
        ty = lineitem['y1']
        lineitem['x1'] = lineitem['x2']
        lineitem['y1'] = lineitem['y2']
        lineitem['x2'] = tx
        lineitem['y2'] = ty
      end
    else
      vec1 = PPoint.new(lineitem['x1']-x, lineitem['y1']-y)
      vec2 = PPoint.new(lineitem['x2']-x, lineitem['y2']-y)
      if vec2.length < vec1.length then
        tx = lineitem['x1']
        ty = lineitem['y1']
        lineitem['x1'] = lineitem['x2']
        lineitem['y1'] = lineitem['y2']
        lineitem['x2'] = tx
        lineitem['y2'] = ty
      end
    end
    return match
  end

  def add_seq_move(lineitem)
    if ((lineitem['x1'] - @position_x).abs < 0.000001) &&
       ((lineitem['y1'] - @position_y).abs < 0.000001) &&
       (@is_home == false) then
      pos = {}
      pos['g'] = 'G01'
      pos['x'] = lineitem['x2']
      pos['y'] = lineitem['y2']
      pos['z'] = @position_z
      @seqdata << pos
    else
      pos = {}
      pos['g'] = 'G01'
      pos['x'] = @position_x
      pos['y'] = @position_y
      pos['z'] = @float_height
      @seqdata << pos
      pos = {}
      pos['g'] = 'G00'
      pos['x'] = lineitem['x1']
      pos['y'] = lineitem['y1']
      pos['z'] = @float_height
      @seqdata << pos
      pos = {}
      pos['g'] = 'G01'
      pos['x'] = lineitem['x1']
      pos['y'] = lineitem['y1']
      pos['z'] = @position_z
      @seqdata << pos
      pos = {}
      pos['g'] = 'G01'
      pos['x'] = lineitem['x2']
      pos['y'] = lineitem['y2']
      pos['z'] = @position_z
      @seqdata << pos
    end
    @position_x = lineitem['x2']
    @position_y = lineitem['y2']
    @is_home = false
  end

  def prepare_seq
    pos = {}
    pos['g'] = 'G00'
    pos['x'] = 0.0
    pos['y'] = 0.0
    pos['z'] = @float_height
    @seqdata << pos
    @is_home = true
  end

  def finish_seq
    pos = {}
    pos['g'] = 'G01'
    pos['x'] = @position_x
    pos['y'] = @position_y
    pos['z'] = @float_height
    @seqdata << pos
    pos = {}
    pos['g'] = 'G00'
    pos['x'] = 0.0
    pos['y'] = 0.0
    pos['z'] = @float_height
    @seqdata << pos
    @position_x = 0.0
    @position_y = 0.0
    @is_home = true
  end

  def build_path
    @seqdata.clear
    prepare_seq
    while @position_z > @mil_depth do
      # -- depth
      if (@mil_depth - @position_z) < @layer_height then
        @position_z += @layer_height
      else
        @position_z = @mil_depth
      end
      # -- path
      workline = []
      @linedata.each do |item|
        citem = {}
        citem['x1'] = item['x1'] - @offset_x
        citem['y1'] = item['y1'] - @offset_y
        citem['x2'] = item['x2'] - @offset_x
        citem['y2'] = item['y2'] - @offset_y
        workline << citem
      end
      # make path
      while workline.size > 0 do
        index = match_line(workline, @position_x, @position_y)
        if index == -1 then
          index = nearest_line(workline, @position_x, @position_y)
        end
        lineitem = workline.at(index)
        workline.delete_at(index)
        align_line(lineitem, @position_x, @position_y)
        add_seq_move(lineitem)
      end
      finish_seq
    end
    @is_build = true
  end

  def save(filename)
    if @is_build == false then
      build_path
    end
    open(filename, "w") do |fp|
      printf(fp, "O1234\n")
      printf(fp, "T1M06\n")
      printf(fp, "G90G54\n")
      printf(fp, "F%.3f\n", @mil_feed)
      printf(fp, "S3600\n")
      printf(fp, "G00Z%f\n", @float_height)
      printf(fp, "M03\n")
      #
      @seqdata.each do |item|
        printf(fp, "%sX%2.4fY%2.4fZ%2.4f\n",
               item['g'], item['x'], item['y'], item['z'])
      end
      #
      printf(fp, "M05\n")
      printf(fp, "M30\n")
      printf(fp, "%\n")
    end
  end

end


class GCodeDrill < GCodeFormatter
  def initialize
    super
    @layer_height = -5.0
    @mil_depth = -2.0
    @mil_feed = 50
    @tool_set = []
  end

  def load(filename)
    drill_size = 0
    @seqdata.clear
    open(filename, "r") do |fp|
      prepare_seq
      fp.each do |line|
        if line=~ /^T([-0-9]+)/ then
          drill_size = $1.to_i
          if @tool_set.include?(drill_size) == false then
            @tool_set << drill_size
          end
        end
        if line=~ /^X([-0-9]+)Y([-0-9]+)/ then
	  a = $1
    	  b = $2
	  @position_x = (a.to_f / 100) - @offset_x
	  @position_y = (b.to_f / 100) - @offset_y
      	  pos = {}
      	  pos['g'] = 'G00'
      	  pos['x'] = @position_x
      	  pos['y'] = @position_y
      	  pos['z'] = @float_height
      	  pos['tool'] = drill_size
          @seqdata << pos
      	  pos = {}
      	  pos['g'] = 'G01'
      	  pos['x'] = @position_x
      	  pos['y'] = @position_y
      	  pos['z'] = @mil_depth
      	  pos['tool'] = drill_size
          @seqdata << pos
      	  pos = {}
      	  pos['g'] = 'G01'
      	  pos['x'] = @position_x
      	  pos['y'] = @position_y
      	  pos['z'] = @float_height
      	  pos['tool'] = drill_size
          @seqdata << pos
        end
      end
      finish_seq
      @is_build = true
    end
  end

  def save(filename)
    @tool_set.each do |tsize|
      workseq = []
      @seqdata.each do |point|
        if point['tool'] == tsize then
          workseq << point
        end
      end
      #
      dotsp = filename.split('.')
      last = dotsp.size
      if last > 1 then
        dotsp[last-2] = sprintf("%s_t%02d", dotsp[last-2], tsize) 
      end
      outputname = dotsp.join('.')
      output(outputname, workseq)
    end
  end

  def output(filename, seqdata)
    open(filename, "w") do |fp|
      printf(fp, "O1234\n")
      printf(fp, "T1M06\n")
      printf(fp, "G90G54\n")
      printf(fp, "F%.3f\n", @mil_feed)
      printf(fp, "S3600\n")
      printf(fp, "G00Z%f\n", @float_height)
      printf(fp, "M03\n")
      #
      seqdata.each do |item|
        printf(fp, "%sX%2.4fY%2.4fZ%2.4f\n",
               item['g'], item['x'], item['y'], item['z'])
      end
      #
      printf(fp, "M05\n")
      printf(fp, "M30\n")
      printf(fp, "%\n")
    end
  end

end
